<?php
/**
 * LehrevaluationsPlugin.class.php
 *
 * Diese Datei enthlt die Hauptklasse des Plugins
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax+studip@gmail.com>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright IBIT 2006-2009
 * @version 2.1
 */

//Imports
require_once($GLOBALS['PLUGINS_PATH'].'/IBIT/LehrevaluationsPlugin/config.inc.php');

/**
 * Die Hauptklasse des Plugins
 *
 * Diese Klasse stellt die Hauptklasse des Plugins dar. Sie bernimmt dabei die Funktion eines FrontControllers, was bedeutet, dass alle
 * Anfragen an diese Klasse gesendet werden. Die Klasse selbst wird die Verarbeitung der Anfragen dann an andere Klassen weiterleiten.
 * Die Initialisierung des Plugins geschieht ebenfalls hier.
 *
 * @author     Jan-Hendrik Willms <tleilax+studip@gmail.com>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author     Ivn Marcos <ivan.marcos@gmx.net>
 * @package    IBIT_Evaluation
 * @subpackage Plugin
 */
class LehrevaluationsPlugin extends AbstractStudIPSystemPlugin
{
	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 */
	public  $template_factory;
	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 */
	private $admin_view = false;
	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 */
	private $teach_view = false;
	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 */
	private $student_view = false;

	/**
	 * Konstruktor des Plugins.
	 * @uses PluginNavigation
	 * @todo Installationsskript einbauen (berprfen, ob vorhanden; wenn nicht - Beispielfragebogen erstellen, Rolle einfgen und Verzeichnis-Rechte setzen
	 */
	public function __construct()
	{
		parent::AbstractStudIPSystemPlugin();
		$this->setPluginiconname('images/chart_pie_edit.png');

		// Templateengine laden
		$this->template_factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');

		# Navigationsreiter erzeugen
		$nav = new PluginNavigation();
		$nav->setDisplayname($this->getPluginname());
		$this->setNavigation($nav);
	}

	/**
	 * Setzt den Pfad zum Plugin.
	 * @param string $path Pfad zum Plugin
	 */
	function setPluginpath( $path )
	{
		parent::setPluginpath( $path );
		$this->buildMenu();
	}

	/**
	 * Erstellt das Men des Plugins.
	 * @uses PluginNavigation
	 */
	public function buildMenu()
	{
		$tab = new PluginNavigation();
		$tab->setDisplayname(_('Lehr-Evaluationen'));
		$this->setNavigation( $tab );
	}

	/**
	 * Liefert den Namen des Plugins zurck.
	 * @return string Der Name des Plugins
	 */
	public function getPluginname()
	{
		return _('Lehrveranstaltungs-Evaluationen');
	}

	/**
	 * Erstellt ein Templateobjekt
	 *
	 * @param string $template_file Dateiname des Templates
	 * @return template Objekt
	 */
	public function getTemplate($template_file)
	{
		$template = $this->template_factory->open($template_file);
		//echo "temlpate erzeugt?!? ";
		//print_r($template);
		$template->set_attribute('link_get_confirmation', PluginEngine::getLink($this, array('section'=>$section, 'confirm'=>'this')));
		$template->set_attribute('link_main', PluginEngine::getLink($this));
		$template->set_attribute('link_base', PluginEngine::getLink($this, array('section'=>$section)));
		$template->set_attribute('evaluation_summary_minusers', ibit_EvaluationConfig::GetConfig('evaluation', 'min_users'));
		$template->set_attribute('admin_view', $this->admin_view);
		$template->set_attribute('teacher_view', $this->teacher_view);
		return $template;
	}

	/**
	 * Stellt die Seite dar, indem der Header aus der Sitemap erzeugt wird und der Content in das Template eingefgt wird.
	 *
	 * @param array $sitemap Individuelle Sitemap des jeweiligen Benutzertyps (admin, dozent, student)
	 * @param string $content Inhalt der Seite
	 * @uses PluginEngine::getLink()
	 */
	public function displayPage( $sitemap, $content )
	{
		# Header erzeugen
		$current_section = empty( $_REQUEST['section'] ) ? $sitemap[0]['subitems'][0]['section'] : $_REQUEST['section'];
		$current_function = empty( $_REQUEST['function'] ) ? $sitemap[0]['subitems'][0]['function'] : $_REQUEST['function'];

		$max_idx = count($sitemap)-1;
		$outer_tab_image = 2;

		$sites = array();
		foreach ( $sitemap as $idx=>$item )
		{
			$section = array( 'title'=>$item['title'], 'link'=>PluginEngine::getLink($this, (array)$item['link_params']), 'current_section'=> false, 'sites'=>array() );
			$current = false;
			foreach ( $item['subitems'] as $subitem )
			{
				if($subitem['section'] == $current_section)
				{
					 $section['current_section'] = true;
				}
				$params = array( 'section'=>$subitem['section'] );
				if ( !empty( $subitem['function'] ) )
				{
					$params['function'] = $subitem['function'];
				}
				if ( !empty( $subitem['params'] ) )
				{
					$params = array_merge( $params, $subitem['params'] );
				}
				$link = PluginEngine::getLink( $this, $params );
				$site = array( 'title'=>$subitem['title'], 'link'=>$link, 'hidden'=>!empty($subitem['hidden']) );
				if ( empty( $subitem['function'] ) and empty( $_REQUEST['function'] ) )
				{
					$current = ( $site['current'] = ( $subitem['section'] == $current_section ) ) || $current;
				}
				elseif ( !empty( $subitem['function'] ) )
				{
					$current = ( $site['current'] = ( $subitem['section'] == $current_section and $subitem['function'] == $current_function ) ) || $current;
				}
				elseif($this->student_view)
				{
					$current = 1;
				}
				array_push( $section['sites'], $site );
			}

			$tab_image = 1;
			if ( $section['current'] = $current )
			{
				if ( $idx == 0 )
				{
					$outer_tab_image = 1;
				}
				$tab_image = 2;
			}

			$section[ 'tab_image' ] = ( $idx == $max_idx ) ? 4 : $tab_image;

			array_push( $sites, $section );
		}

		$template = $this->getTemplate('common_header');
		$template->set_attribute('tab_image', $outer_tab_image);
		$template->set_attribute('sites', $sites);
		$template->set_attribute('page_content', $content);
		$template->set_attribute('error_messages', $this->error_messages);
		$template->set_attribute('success_messages', $this->success_messages);
		echo $template->render();
		return;

	}

	/**
	 * Hauptanzeigefunktion des Plugins.
	 *
	 * ber diese Funktion werden alle Seiten des Plugins verwaltet und ausgegeben.
	 * @uses ibit_Action_Admin
	 * @uses ibit_Action_Teacher
	 * @uses ibit_Action_User
	 */
	public function actionshow()
	{
		$GLOBALS['ibit_eval_pluginpath'] = $this->getPluginURL();

		// Login prfen
		if (!isset($_SESSION['auth']) or !is_object($_SESSION['auth']) or !isset($_SESSION['auth']->auth) or !isset($_SESSION['auth']->auth['uid']) or ($_SESSION['auth']->auth['uid']=='nobody'))
		{
			echo Messagebox::error('Sie mssen eingeloggt sein, um dieses Plugin aufrufen zu drfen');
			return;
		}

		// ?!?
		$section = empty( $_REQUEST['section'] ) ? '' : trim( $_REQUEST['section'] );
		if ( !empty( $_REQUEST['confirm'] ) and $_REQUEST['confirm'] == 'this' )
		{
			$params = array();
			if ( !empty( $section ) )
			{
				$params['section'] = $section;
			}

			$template = $this->getTemplate('common_confirm');
			$template->set_attribute('confirm_message', urldecode($_REQUEST['message']));
			$template->set_attribute('confirm_yes', urldecode($_REQUEST['yes']));
			$template->set_attribute('confirm_no', urldecode($_REQUEST['no']));
			echo $template->render();
			return;

		}

		# Falls Druckansicht, nichts anzeigen von studip
		if(!empty($_GET['print']) and $_GET['print']=='yes')
		{
			ob_end_clean();
		}
		ob_start();

		$user = $this->getUser();
		$permission	= $user->getPermission();

		# Prfen, ob der User die Rolle 'eval_admin' hat oder Fakulttsadmin ist
		$eval_admin = false;
		$roles = $user->getAssignedRoles();
		foreach ($roles as $role)
		{
			if ($role->getRolename() == 'eval_admin')
			{
				$eval_admin = true;
			}
		}

		# Root ist eingeloggt
		if ($permission->hasRootPermission() or $eval_admin)
		{
			$sitemap = $GLOBALS['evaluation_menu_admin'];
			$this->admin_view = true;
			new ibit_Action_Handler_Admin($this, $user->userid);
		}

		# Dozent ist eingeloggt
		elseif($permission->hasTeacherPermission())
		{
			$sitemap = $GLOBALS['evaluation_menu_dozent'];
			$this->teacher_view = true;
			new ibit_Action_Handler_Teacher( $this, $user->userid );
		}
		# Student ist eingeloggt
		else
		{
			$sitemap = $GLOBALS['evaluation_menu_student'];
			$this->student_view = true;
			new ibit_Action_Handler_User( $this, $user->userid );
		}

		# Falls keine Druckansicht
		if(empty($_GET['print']))
		{
			$content = ob_get_clean();
			$this->displayPage($sitemap, $content);
		}
	}
}
?>