<?php
/**
 * classes/ibit_Action_EvaluationActivationSeminar.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006, 2007 IBIT
 * @version 2.0.7
 */

/**
 * Die Evaluationsaktivierung erzeugt eine Evaluation fr ein einzelnes Seminar aus einer Vorlagen.
 *
 * Diese Klasse bernimmt das Aktivieren von Evaluationen bzw. das Umsetzen von Vorlagen in konkrete Evaluationen fr einzelne Seminare.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Action
 * @copyright  Copyright 2006, 2007 IBIT
 * @version    2.0.7
 */
class ibit_Action_EvaluationActivationSeminar
{
	/**
	 * Referenz auf das aufrufende Objekt
	 * @var objekt $parent
	 */
	var $parent;

	/**
	 * Konstruktor der Klasse.
	 * @param object $parent Referenz auf das aufrufende Objekt
	 */
	function ibit_Action_EvaluationActivationSeminar($parent)
	{
		$this->parent = $parent;
	}

	/**
	 * Zeigt das Men zur Suche an.
	 * @uses ibit_Helper::getSemester()
	 * @uses ibit_EvaluationSeminar::searchSeminarByNameAndSemester();
	 */
	function show()
	{
		$template = $this->parent->parent->getTemplate('admin_evaluation_activation_seminar_search');
		$template->set_attribute('semester_data', ibit_Helper::getSemester());
		$template->set_attribute('link_accept', $this->parent->base_link.'&command=searched');
		$template->set_attribute('link_cancel', $this->parent->parent_link);
		$template->set_attribute('link_search', $this->parent->base_link);

		if (!empty($_POST))
		{
			if ((strlen($_POST['name'])<3) and (strlen($_POST['number'])<3))
			{
				echo Messagebox::error('Der Suchbegriff muss mindestens 3 Zeichen lang sein' );
			}
			else
			{
				$seminar_data = ibit_EvaluationSeminar::searchSeminarByNameAndSemester($_POST['name'], $_POST['number'], $_POST['semester_id']);
				$template->set_attribute('seminar_data', $seminar_data);
				$template->set_attribute('semester_id', $_POST['semester_id']);
			}
		}
		echo $template->render();
	}

	/**
	 * Zeigt das Men zum Aktivieren an.
	 * @uses ibit_EvaluationSeminar::getSeminarData()
	 * @uses ibit_EvaluationTemplate::getTemplatesInfo()
	 * @uses ibit_Helper::getSemester()
	 */
	function searched()
	{
		if (empty($_POST['seminar_id']))
		{
			echo Messagebox::error('Bitte ein gltiges Seminar auswhlen' );
			$this->show();
			return;
		}
		$template_infos = ibit_EvaluationTemplate::getTemplatesInfo( true );

		$template = $this->parent->parent->getTemplate('admin_evaluation_activation_seminar');
		$template->set_attribute('semester_id', $_POST['semester_id']);
		$template->set_attribute('seminar_data', array_pop(ibit_EvaluationSeminar::getSeminarData( $_POST['seminar_id'])));
		$template->set_attribute('template_infos', $template_infos);
		$template->set_attribute('link_accept', $this->parent->base_link.'&command=activate');
		$template->set_attribute('link_cancel', $this->parent->parent_link);
		$template->set_attribute('link_preview', $this->parent->parent_link.'?section=templates&function=preview');
		echo $template->render();
	}

	/**
	 * Aktiviert eine Vorlage fr ein Seminar.
	 * @uses ibit_Evaluation
	 * @uses ibit_Helper::checkNumber()
	 * @uses ibit_EvaluationSeminar::getSeminarData()
	 * @uses ibit_Helper::protectTemplate()
	 */
	function activate()
	{
		$error = false;
		# Tag berprfen
		ibit_Helper::checkNumber($_POST['start_day'], 1, 31, 'Start-Tag', $error);
		ibit_Helper::checkNumber($_POST['end_day'], 1, 31, 'Ende-Tag', $error);
		# Monat berprfen
		ibit_Helper::checkNumber($_POST['start_month'], 1, 12, 'Start-Monat', $error);
		ibit_Helper::checkNumber($_POST['end_month'], 1, 12, 'Ende-Monat', $error);
		# Jahr berprfen
		ibit_Helper::checkNumber($_POST['start_year'], 2000, null, 'Start-Jahr', $error);
		ibit_Helper::checkNumber($_POST['end_year'], 2000, null, 'Ende-Jahr', $error);
		# Template-Id berprfen
		if (empty($_POST['activation_template_id']))
		{
			echo Messagebox::error('Bitte Evaluationsvorlage whlen');
			$error = true;
		}
		# Semester-Id berprfen
		if (empty($_POST['semester_id']))
		{
			echo Messagebox::error('Bitte gltiges Semester whlen');
			$error = true;
		}
		# Institut-Ids berprfen
		if (empty($_POST['seminar_id']))
		{
			echo Messagebox::error('Bitte ein gltiges Seminar auswhlen');
			$error = true;
		}

		if ($error == true)
		{
			$this->show();
			return;
		}

		$start_time = mktime(0,  0,  0, $_POST['start_month'], $_POST['start_day'], $_POST['start_year']);
		$end_time = mktime(23, 59, 59, $_POST['end_month'], $_POST['end_day'], $_POST['end_year']);
		$eval_template_id = $_POST['activation_template_id'];
		$semester_id = $_POST['semester_id'];
		$seminar_id = $_POST['seminar_id'];

		$seminar_info = array_pop(ibit_EvaluationSeminar::getSeminarData($seminar_id));

		$evaluation = new ibit_Evaluation();
		$evaluation->setTemplateId($eval_template_id);
		$evaluation->setSeminarId($_POST['seminar_id']);
		$evaluation->setInstitutId($seminar_info['Institut_id']);
		$evaluation->setFakultaetId($seminar_info['Fakultaet_id']);
		$evaluation->setStatus(1);
		$evaluation->setConfirmed(0);
		$evaluation->setSemesterId($semester_id);
		$evaluation->setStartDate($start_time);
		$evaluation->setStopDate($end_time);
		$evaluation->setCreateAuthorId($_SESSION['auth']->auth['uid']);
		$evaluation->setChangeAuthorId($_SESSION['auth']->auth['uid']);
		$evaluation->save();

		ibit_Helper::protectTemplate($eval_template_id);

		// Insert tutors into user table
		$tutor_ids = ibit_EvaluationSeminar::getTutorsForSeminar($_POST['seminar_id']);
		foreach ($tutor_ids as $tutor_id)
		{
			DBManager::Get()->query("INSERT INTO ibit_evaluation_user (evaluation_id, user_id, timestamp, hidden) VALUES ('{$eval_template_id}', '{$tutor_id}', NOW(), 1)");
		}

		echo Messagebox::success('Die Evaluation fr das Seminar wurde erfolgreich eingetragen');
		$this->parent->evaluation_administration();
	}
}
?>