<?php
/**
 * classes/ibit_Action_EvaluationXmlExport.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006, 2007 IBIT
 * @version 2.0.4
 */

/**
 * Diese Klasse enthlt die Logik zum XML-Export.
 *
 * In dieser Klasse wird die Logik des XML-Exportes nach SAP verwaltet. Die Evaluationsdaten des ausgewhlten Semesters werden
 * als Flat-XML-Dateien wahlweise in einem Ordner gespeichert oder zum Download angeboten. Bei der Auswahl der Evaluationen wird
 * drauf geachtet, ob diese vom Dozenten freigeschaltet wurden und die angegebene Mindestanzahl an Teilnehmern erreicht wurde.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  	Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Action
 * @copyright  Copyright 2006, 2007 IBIT
 * @version    2.0.4
 */
class ibit_Action_EvaluationXmlExport
{
	/**
	 * Enthlt eine Referenz auf das aufrufende Objekt.
	 * @var object $parent Referenz auf das aufrufende Objekt
	 */
	var $parent;


	/**
	 * Konstruktor
	 * @param object $parent Referenz auf das aufrufende Objekt
	 */
	function ibit_Action_EvaluationXmlExport( $parent )
	{
		$this->parent = $parent;
	}

	/**
	 * Zeigt das Hauptmen der Export-Funktion.
	 * @uses ibit_Helper::getEvaluationSemester()
	 */
	function showMenu()
	{
		$template = $this->parent->parent->getTemplate('admin_xml_export_menu.php');
		$template->set_attribute('min_users', ibit_EvaluationConfig::GetConfig('evaluation', 'min_users'));
		$template->set_attribute('semester', ibit_Helper::getEvaluationSemester(true));
		$template->set_attribute('link_export', $this->parent->base_link.'&command=export');
		echo $template->render();
	}

	/**
	 * Fhrt den eigentlichen Export aus.
	 * @uses ibit_Helper::getSemesterName()
	 * @uses ibit_Evaluation::getEvaluationAnswers()
	 * @uses ibit_EvaluationSeminar::getSeminarData()
	 * @uses ibit_EvaluationSeminar::getDozentDataForSeminar();
	 * @uses XML_Writer
	 */
	function export()
	{
		if ($_POST['semester_id']==-1)
		{
			echo Messagebox::error('Bitte ein gltiges Semester auswhlen');
			$this->showMenu();
			return;
		}

		# /export-Pfad im Tempverzeichnis anlegen, falls er nicht existiert
		if (!(is_dir( $GLOBALS['TMP_PATH'].'/export') or @mkdir($GLOBALS['TMP_PATH'].'/export')))
		{
			echo Messagebox::error('Fehler beim Erstellen des Export-Verzeichnisses');
			return;
		}

		# Aktuelles Export-Verzeichnis ermitteln und anlegen, falls nicht vorhanden
		$path_name = preg_replace('~\s|/~', '', ibit_Helper::getSemesterName($_POST['semester_id']));
		$date = date('Y-m-d');
		$export_path =   $GLOBALS['TMP_PATH'].'/export/'.$path_name.'_'.$date;

		if (!(is_dir( $export_path) or @mkdir($export_path)))
		{
			echo Messagebox::error('Fehler beim Erstellen des Export-Verzeichnisses');
			return;
		}
		else
		# Pfad leeren
		{
			$files = glob($export_path.'/*');
			array_map('unlink', $files);
		}

		# Freischaltungseinstellungen abfragen
		$visible = !empty($_POST['only_visible']);

		# Informationen sammeln
		$evaluation_ids = ibit_Evaluation::getEvaluationExportIDs($_POST['semester_id'], $_POST['min_user'], $visible);

		if (empty($evaluation_ids))
		{
			echo Messagebox::error('Es gibt keine Evaluation in diesem Semester, die den Anforderungen entsprechen');
			$this->showMenu();
			return;
		}

		# Gruppen, Fragen, Antworten und Userantworten ausgeben
		$seminar_ids = array();
		$evaluation_seminar_data = array();

		$group_xml = new XML_Writer($export_path.'/evalgroups', 'root', 'row', true, true);
		$question_xml = new XML_Writer($export_path.'/evalquestions', 'root', 'row', true, true);
		$answer_xml = new XML_Writer($export_path.'/evalanswers', 'root', 'row', true, true);
		$useranswer_xml = new XML_Writer($export_path.'/evaldata', 'root', 'row', true, true);
		if (!empty($_POST['max_filesize']) and $_POST['max_filesize']!=-1)
		{
			$group_xml->setMaxSize($_POST['max_filesize']);
			$question_xml->setMaxSize($_POST['max_filesize']);
			$answer_xml->setMaxSize($_POST['max_filesize']);
			$useranswer_xml->setMaxSize($_POST['max_filesize']);
		}

		$random_id = 1;
		foreach ($evaluation_ids as $evaluation_id)
		{
			$evaluation = ibit_Evaluation::getEvaluationAnswers($evaluation_id);

			array_push($seminar_ids, $evaluation['seminar_id']);
			array_push($evaluation_ids, $evaluation['id']);
			array_push($evaluation_seminar_data, array(
				'evaluation' => $evaluation['id'],
				'seminar' => $evaluation['seminar_id']
			));
			foreach ($evaluation['groups'] as $eval_group)
			{
				$group = array(
					'title' => $eval_group['title'],
					'text' => $eval_group['description'],
					'position' => $eval_group['position'],
					'evalgroup_id' => $eval_group['group_id']
				);

				$group_xml->saveRow($group);

				foreach ($eval_group['questions'] as $eval_question)
				{
					$question = array(
						'type' => $eval_question['answer_title'],
						'position' => $eval_question['position'],
						'title' => $eval_question['title'],
						'text' => $eval_question['text'],
						'evalquestion_id' => $eval_question['question_id'],
						'evalgroup_id' => $eval_group['group_id']
					);

					$question_xml->saveRow($question);

					foreach ($eval_question['answers'] as $eval_answer)
					{
						$answer = array(
							'evalanswer_id' => $eval_answer['answer_id'],
							'position' => $eval_answer['position'],
							'text' => $eval_answer['text'],
							'value' => ($eval_answer['value']=='-1' ? 'k.A.' : $eval_answer['value']),
							'evalquestion_id' => $eval_question['question_id']
						);

						$answer_xml->saveRow($answer);

						$useranswer = array(
							'Fakultaets_id' => $evaluation['fakultaet_id'],
							'Institut_id' => $evaluation['institut_id'],
							'Seminar_id' => $evaluation['seminar_id'],
							'evalgroup_id' => $eval_group['group_id'],
							'evalquestion_id' => $eval_question['question_id'],
							'evalanswer_id' => $eval_answer['answer_id'],
							'value' => strip_tags(str_replace(array("\r","\n"), ' ', $eval_answer['user_value'])),
							'Anzahl' => $eval_answer['count'],
							'Random_id' => $random_id++
						);

						if ($useranswer['value']==-1)
						{
							$useranswer['value'] = 'k.A.';
						}

						if (!empty($_POST['strip_text']) and $_POST['strip_text']!=-1 and strlen($useranswer['value'])>$_POST['strip_text'])
						{
							$useranswer['value'] = substr($useranswer['value'], 0, $_POST['strip_text'] - 3).'...';
						}

						$useranswer_xml->saveRow($useranswer);
					}
				}
			}
			unset($evaluation);
		}
		$group_xml->close();
		$question_xml->close();
		$answer_xml->close();
		$useranswer_xml->close();

		# Evaluationsteilnehmer pro Seminar bestimmen
		$evaluation_participants = ibit_Evaluation::getEvaluationParticipants( $evaluation_ids );
		$seminar_participants = Array();
		foreach ($evaluation_seminar_data as $item)
		{
			$seminar_participants[$item['seminar']] = empty($evaluation_participants[$item['evaluation']])?0:$evaluation_participants[$item['evaluation']];
		}

		# Seminare ausgeben
		$seminare = ibit_EvaluationSeminar::getSeminarData($seminar_ids);

		$seminar_xml = new XML_Writer($export_path.'/evalseminare', 'root', 'row', true, true);
		if (!empty($_POST['max_filesize']) and $_POST['max_filesize']!=-1)
		{
			$seminar_xml->setMaxSize( $_POST['max_filesize'] );
		}
		foreach ($seminare as $seminar)
		{
			$participants = empty($seminar_participants[$seminar['Seminar_id']])?0:$seminar_participants[$seminar['Seminar_id']];
			$seminar['Fakultaets_id'] = ibit_Evaluation::getFacultyForSeminar($seminar['Seminar_id']);

			$seminar['Teilnehmer_Evaluation'] = $participants;
			$seminar_xml->saveRow($seminar);
		}
		$seminar_xml->close();

		# Dozenten ausgeben
		$dozenten = ibit_EvaluationSeminar::getDozentDataForSeminar($seminar_ids);

		$dozent_xml = new XML_Writer($export_path.'/evaldozenten', 'root', 'row', true, true);
		if (!empty($_POST['max_filesize']) and $_POST['max_filesize']!=-1)
		{
			$dozent_xml->setMaxSize($_POST['max_filesize']);
		}
		foreach ($dozenten as $dozent)
		{
			if (empty($seminare[$dozent['seminar_id']]))
			{
				$dozent['Institut_id'] = 'unbekannt';
			}
			else
			{
				$dozent['Institut_id'] = $seminare[ $dozent['seminar_id'] ]['Institut_id'];
			}
			$dozent_xml->saveRow($dozent);
		}
		$dozent_xml->close();

		# Allgemeine Infos ausgeben
		$evaluation_stats = ibit_Evaluation::getEvaluationStatisticsForSemester($_POST['semester_id']);

		$stats_xml = new XML_Writer($export_path.'/evalstats', 'root', 'row', true, true);
		foreach ($evaluation_stats as $faculty_id=>$data)
		{
			foreach ($data as $institute_id=>$statistics)
			{
				$statistics['Fakultaets_id'] = empty($faculty_id)?'Summe':$faculty_id;
				$statistics['Institut_id'] = empty($institute_id)?'Summe':$institute_id;
				$stats_xml->saveRow($statistics);
			}
		}
		$stats_xml->close();

		# Prfen, ob die Datei heruntergeladen werden soll
		#print_r($_POST);
		#if (isset($_POST['export']))
		#{
		#	echo "herunterladen = false";
		#	echo Messagebox::success('Die XML-Dateien wurden erfolgreich in das Verzeichnis <b>'.$export_path.'</b> exportiert');
		#	return;
		#}

		# Erzeugte Dateien zu einem Zip-File packen
		$zipfilename = 'Evaluation_'.$path_name.'_'.$date.'.zip';
		$zipped = create_zip_from_directory($export_path, $GLOBALS['TMP_PATH'].'/export/'.$zipfilename);
		if ($zipped===false)
		{
			echo Messagebox::error('Fehler beim Packen der XML-Dateien');
			return;
		}
		# Erzeugte Dateien inklusive Pfad lschen
		$files = glob($export_path.'/*');
		array_map('unlink', $files);
		rmdir($export_path);

		# Erfolgsmeldung inklusive Downloadlink ausgeben
		echo Messagebox::success('Das ZIP-File kann <a href="'.getDownloadLink($zipfilename, $zipfilename, 2, 'force').'">hier</a> heruntergeladen werden.');
	}
}
?>