<?php
/**
 * classes/ibit_EvaluationSummary.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006-2010 IBIT
 * @version 2.2
 */

/**
 *
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Structure
 * @copyright  Copyright 2006-2010 IBIT
 * @version    2.0.2
 *
 */
class ibit_EvaluationSummary
{
	/**
	 * Die ID der Evaluation
	 * @var string $evaluation_id
	 */
	var $evaluation_id = null;

	var $admin_id = null;

	var $cache_key = null;

	var $min_users = null;

    /**
     * Konstruktor
     */
    function ibit_EvaluationSummary( $evaluation_id = null , $admin_id = null )
    {
    	$this->evaluation_id = $evaluation_id;
    	$this->admin_id = $admin_id;
    	$this->cache_key = md5('auswertung'.$this->evaluation_id);
    	$this->min_users = ibit_EvaluationConfig::GetConfig('evaluation', 'min_users');
    }

    /**
     * Grunddaten liefern wie Evalname, Teilnehmer, Datum etc. pp.
     */
    function getOverview()
    {
        //Objekt erstellen
        $cache = StudipCacheFactory::getCache();

        //Wert auslesen
        $overview = unserialize($cache->read($this->cache_key.'/overview'));

        // wenn cache leer, daten holen
        if(empty($overview)){

    		# Anzahl der teilnehmenden Studenten an der Evaluation ermitteln
    		$user_count = DBManager::Get()->query("SELECT SUM(hidden<>1) AS user_count FROM ibit_evaluation_user WHERE evaluation_id = '{$this->evaluation_id}'")->fetchColumn();
    		$overview = array();

    		#Daten holen
    		$evaluation = new ibit_Evaluation($this->evaluation_id);
    		$seminar = new ibit_EvaluationSeminar($evaluation->getSeminarId());
    		$template = new ibit_EvaluationTemplate($evaluation->getTemplateId());

    		#Daten ins Ausgabearray speichern.
    		$overview['id']= $this->evaluation_id;
    		$overview['users']= $user_count;
    		$overview['start_date']= $evaluation->getStartDate();
    		$overview['stop_date']= $evaluation->getStopDate();
    		$overview['create_date']= $evaluation->getCreateDate();
    		$overview['comment']= $evaluation->getComment();

    		$overview['template_name'] = $template->getTitle();

    		if($evaluation->isVisible())
    		{
    			$overview['visible_date']= strtotime($evaluation->getVisibleDate());
    			$overview['visible_status']= true;
    		}

    		$overview['seminar_id']= $evaluation->getSeminarId();
    		$overview['seminar_name']= $seminar->getName();
    		$overview['seminar_teachers']= $seminar->getTeachers();
    		$overview['seminar_countteachers']= count($seminar->getTeachers());
    		$overview['seminar_members']= $seminar->getMembers();
    		$overview['participants_ratio']= ($user_count / $seminar->getMembers() * 100);

    		//daten in cache speichern
    		$cache->write($this->cache_key.'/overview', serialize($overview));
        }

        //daten erweitern
        if(!empty($this->admin_id))
        {
            $overview['user_id'] = $this->admin_id;
        }

        //und zurckgeben
		return $overview;
    }

    /**
	 * Ldt die Antworten zu einer Evaluation.
	 * @param int $evaluation_id Id der Evaluation
	 * @param int $users Minimale Anzahl an teilnehmenden Studenten (Rckgabe ist die tatschliche Anzahl der Teilnehmer oder false bei Fehlern)
	 * @return mixed false bei Fehlern oder zu wenig Teilnehmern, ansonsten ein array mit den Daten
	 * @static
	 * @todo Dokumentation, Aggregation
	 */
	function getAnswers()
	{
	    //Objekt erstellen
        $cache = StudipCacheFactory::getCache();

        //Wert auslesen
        $result = unserialize($cache->read($this->cache_key.'/answers'));

        // wenn cache leer, daten holen
        if(empty($result)){

    		$users = true;

    		# Anzahl der teilnehmenden Studenten an der Evaluation ermitteln
    		$user_count = DBManager::Get()->query("SELECT SUM(hidden<>1) AS user_count FROM ibit_evaluation_user WHERE evaluation_id = '{$this->evaluation_id}'")->fetchColumn();
    		if ( $user_count < $users )
    		{
    			$users = $user_count;
    			return false;
    		}
    		$users = $user_count;

    		$result = array();
    		$group_ids = array();
    		$question_ids = array();
    		$answer_ids = array();

    		# Struktur laden
    		$rows = DBManager::Get()->query("SELECT evaluation_group_template_id, q.evaluation_question_template_id, a.evaluation_answer_template_id AS evaluation_answer_template_id, a.text AS answer_text, a.value AS answer_value, group_position, question_position, a.position AS answer_position FROM ibit_evaluation_links AS l INNER JOIN ibit_evaluation AS e USING( evaluation_template_id ) LEFT JOIN ibit_evaluation_question_templates AS q ON (l.evaluation_question_template_id=q.evaluation_question_template_id) LEFT JOIN ibit_evaluation_answer_templates AS a ON (q.evaluation_answer_type_id=a.evaluation_answer_type_id) WHERE evaluation_id = '{$this->evaluation_id}' ORDER BY group_position ASC, question_position ASC")->fetchAll(PDO::FETCH_ASSOC);

    		if ( empty($rows) )
    		{
    			return false;
    		}
    		foreach ($rows as $row)
    		{
    			$group_id = $row['evaluation_group_template_id'];
    			$group_position = $row['group_position'];
    			$question_id = $row['evaluation_question_template_id'];
    			$question_position = $row['question_position'];
    			$answer_id = $row['evaluation_answer_template_id'];
    			$answer_position = $row['answer_position']+1;
    			$answer_text = $row['answer_text'];
    			$answer_value = $row['answer_value'];

    			if ( empty( $result[ $group_id ] ) )
    			{
    				$result[ $group_id ] = array( 'group_id' => $group_id, 'position' => $group_position, 'questions' => array() );
    				array_push( $group_ids, $group_id );
    			}
    			if ( empty( $result[ $group_id ]['questions'][ $question_id ] ) )
    			{
    				$result[ $group_id ]['questions'][ $question_id ] = array( 'question_id' => $question_id, 'position' => $question_position, 'answers' => array() );
    				array_push( $question_ids, $question_id );
    			}
    			$result[ $group_id ]['questions'][ $question_id ]['answers'][ $answer_id ] = array(
    				'answer_id' => $answer_id,
    				'position' => $answer_position,
    				'count' => 0,
    				'text' => $answer_text,
    				'value' => $answer_value,
    				'values' => array()
    			);
    		}

    		# Antworten auslesen
    		$rows = DBManager::Get()->query("SELECT evaluation_group_template_id, evaluation_question_template_id, evaluation_answer_template_id, TRIM(value) AS value, COUNT(*) AS value_count FROM ibit_evaluation_answers WHERE evaluation_id = '{$this->evaluation_id}' GROUP BY evaluation_group_template_id, evaluation_question_template_id, evaluation_answer_template_id, LOWER(TRIM(value))")->fetchAll(PDO::FETCH_ASSOC);
    		if ( empty($rows) )
    		{
    			return false;
    		}
    		foreach ($rows as $row)
    		{
    			$group_id = $row['evaluation_group_template_id'];
    			$question_id = $row['evaluation_question_template_id'];
    			$answer_id = $row['evaluation_answer_template_id'];
    			$value = $row['value'];
    			$value_count = $row['value_count'];

    			$answer = array();
    			$answer['value'] = $value;
    			$answer['count'] = $value_count;
    			$result[ $group_id ]['questions'][ $question_id ]['answers'][ $answer_id ]['count'] += $answer['count'];

    			array_push( $result[ $group_id ]['questions'][ $question_id ]['answers'][ $answer_id ]['values'], $answer );
    		}

    		# Gruppendaten auslesen
    		$rows = DBManager::Get()->query("SELECT evaluation_group_template_id, title, description FROM ibit_evaluation_group_templates WHERE evaluation_group_template_id IN ('".implode("','", array_unique($group_ids))."')")->fetchAll(PDO::FETCH_ASSOC);
    		if ( empty($rows) )
    		{
    			return false;
    		}

    		$group_data = array();
    		foreach ($rows as $row)
    		{
    			$group_data[ $row['evaluation_group_template_id'] ] = $row;
    		}

    		# Fragendaten und Antworttypen auslesen
    		$rows = DBManager::Get()->query("SELECT evaluation_question_template_id, q.title AS title, q.text AS text, t.title AS answer_title, t.description AS answer_description, t.html_type AS html_type FROM ibit_evaluation_question_templates AS q LEFT JOIN ibit_evaluation_answer_types AS t USING( evaluation_answer_type_id ) WHERE evaluation_question_template_id IN ('".implode("','", array_unique($question_ids))."')")->fetchAll(PDO::FETCH_ASSOC);
    		if (empty($rows))
    		{
    			return false;
    		}

    		$question_data = array();
    		foreach ($rows as $row)
    		{
    			$question_data[ $row['evaluation_question_template_id'] ] = $row;
    		}

    		# Daten ins array fllen
    		foreach ( $result as $group_idx => $group )
    		{
    			$group['title'] = $group_data[ $group_idx ]['title'];
    			$group['description'] = $group_data[ $group_idx ]['description'];
    			foreach ( $group['questions'] as $question_idx => $question )
    			{
    				$question['title'] = $question_data[ $question_idx ]['title'];
    				$question['text'] = $question_data[ $question_idx ]['text'];
    				$question['answer_title'] = $question_data[ $question_idx ]['answer_title'];
    				$question['answer_description'] = $question_data[ $question_idx ]['answer_description'];
    				$question['html_type'] = $question_data[ $question_idx ]['html_type'];

    				$answer_count = 0;
    				$answer_countonly = 0;
    				foreach( $question['answers'] as $answer_idx => $answer )
    				{
    					$answer_count = $answer_count + $answer['count'];
    					if($answer['value'] != '-1')
    					{
    						$answer_countonly = $answer_countonly + $answer['count'];
    					}
    				}
    				$question['answer_count'] = $answer_count;
    				$question['answer_countonly'] = $answer_countonly;
    				foreach( $question['answers'] as $answer_idx => $answer )
    				{
    					if($answer_count > 0)
    					{
    						$question['answers'][$answer_idx]['ratio'] = ($answer['count'] / $answer_count) * 100;
    					}
    					if($answer_countonly > 0)
    					{
    						$question['answers'][$answer_idx]['ratioonly'] = ($answer['count'] / $answer_countonly) * 100;
    					}
    				}
    				if($question['html_type'] == 'polskala' || $question['html_type']=='select')
    				{
    					$question['median'] = $this->calculateMedian($question['answers'], $answer_countonly);
    					$question['average'] = $this->calculateAverage($question['answers'], $answer_countonly);
    					$question['deviation'] = $this->calculateDeviation($question['answers'], $answer_countonly, $question['average']);
    				}

    				#GRAFIKTYP AUS DB HOLEN
    				if(!empty($this->admin_id))
    				{
    					$question['chart_type'] = ibit_EvaluationSummaryView::getAdminViewForQuestion( $this->evaluation_id, $group_idx, $question_idx, $this->admin_id );
    				}
    				else
    				{
    					$question['chart_type'] = ibit_EvaluationSummaryView::getUserViewForQuestion( $this->evaluation_id, $group_idx, $question_idx );
    				}
    				$question['imagehash'] = md5($this->evaluation_id.'_'.$question['question_id'].'_'.$question['position'].'_'.$question['chart_type']);

    				#GRAFIK ERZEUGEN
    				if($question['html_type'] != 'text' || $question['html_type']!='textarea')
    				{
    					#echo "Grafik wird erzeugen... ";
    					$graphic = new ibit_Graphics($this->evaluation_id,
    						ibit_EvaluationConfig::GetConfig('image', 'width'),
    						ibit_EvaluationConfig::GetConfig('image', 'height'),
    						ibit_EvaluationConfig::GetConfig('image', 'type'),
    						ibit_EvaluationConfig::GetConfig('image', 'shading')
    					);
    					$graphic->createImage($question);
    					#echo "...fertig.<br/>";
    				}

    				$group['questions'][ $question_idx ] = $question;
    			}
    			$result[ $group_idx ] = $group;
    		}

    		//daten in cache speichern
            $cache->write($this->cache_key.'/answers', serialize($result));
        }

		# Daten zurckgeben
		return $result;
	}

		/**
	 * Mittelwert berechnen
	 *
	 * @param array $answers - Antwortenblock
	 * @return float Mittelwert mit 2 Nachkommastellen
	 */
	function calculateAverage($answers, $total){
		$points = 0;
		$result = 0;
		#echo "berechnung durchschnitt ";
		foreach($answers as $answer){
			#echo " wert: ".$answer['value']." anzahl: ".$answer['count'];
			if($answer['value'] != '-1')
			{
				$points = $points + ($answer['count'] * $answer['value']);
				#echo " berechneter wert: ".$points;
			}
		}
		if($total != 0)
		{
			$result = round($points/$total, 2);
		}
		#echo "<br/>";
		return $result;
	}

	/**
	 * Median berechnen
	 *
	 * @param array $answers - Antwortenblock
	 * @param int $total - Gesamtanzahl Antworten
	 * @return int Median
	 */
	function calculateMedian($answers, $total){
		$before = 0;
		$whole = 0;
		$result = 0;
		$median = round($total/2, 0);
		foreach($answers as $answer){
			if($answer['value'] != '-1'){
				$whole = $whole + $answer['count'];
				if($before < $median && $median <= $whole){
					$result = $answer['value'];

				}
				$before = $before + $answer['count'];
			}
		}
		return $result;
	}

	/**
	 * Standardabweichung berechnen
	 *
	 * @param array $answers - Antwortenblock
	 * @param int $total - Gesamtanzahl Antworten
	 * @param float $average - Mittelwert
	 * @return float $result mit 2 Nachkommastellen
	 */
	function calculateDeviation($answers, $total, $average){
		#echo "gesamtantworten: ".$total;
		if($total != 0){
			$totalpart = 1/($total);
		}
		else
		{
			#echo "Fehler: Keine Antworten vorhanden";
			$totalpart = 0;
		}
		#echo " anteil: ".$totalpart."<br/>";
		$amount = 0;
		$whole = 0;
		foreach($answers as $answer){
			if($answer['value'] != '-1'){
				for($i=0; $i<$answer['count']; $i++){
					$help = pow(($answer['value']-$average), 2);
					$whole = $whole + $help;
				}
			}
		}
		$result = round(sqrt($totalpart * $whole), 2);
		return $result;
	}
}
?>